/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


#include <it.h>
#ifdef IT_WINDOWS
#include <strstrea.h>
#else
#include <strstream.h>
#endif

int
main(int, char **)
{
    ITConnection conn;
    conn.Open();
    ITQuery q(conn);

    // Fetch an array from the Informix database
    ITRow *row =                                                    // 1 begin
        q.ExecOneRow("select * from bitarraytab;");
                                                                    // 1 end
    // Check to see if the row was fetched successfully
    if (q.Error())
    {
	cerr << q.ErrorText() << endl;
	return 1;
    }

    // Check to see if any rows were returned
    if (row == NULL)
    {
        cout << "contain: no rows returned by ITQuery::ExecOneRow" << endl;
        cout << "contain: exiting" << endl;
        return 1;
    }

    // Extract out the first column of the result, which will be the
    // array 
    ITValue *arrayval = row->Column(0);                             // 2 begin
                                                                    // 2 end
    // Get the ITContainCvt interface for array access
    ITContainCvt *arraycont;                                        // 3 begin
    arrayval->QueryInterface(ITContainCvtIID, (void **) &arraycont);
    row->Release();
    arrayval->Release();                                            // 3 end
    
    // The iterator class iterates over every member of an object   // 4 begin
    // exposing an ITContainer or ITContainerCvt interface.
    ITContainerIter iter(arraycont);                                

    // Add all the items to the stream
    char buf[8192];
    ostrstream  cstream( buf, sizeof buf );

    for (int i = 0; i < arraycont->NumItems(); i++)
    {
       int value;
        iter >> value;        
        cstream << '[' << i << ']' << " = " << value << endl;       
    }                                                               // 4 end

    cstream << ends;

    // Check the iterator state; print the result if successfull,
    // else print error message.
    switch( iter.State() )
        {
    default:
        cerr << "Unknown state of the container iterator" << endl;
        break;
    case ITContainerIter::StateOutOfBounds:
        cerr << "Iterator run out of items at " << iter.Index() << endl;
        break;
    case ITContainerIter::StateOK:
        cout << "Iterator successful" << endl; 
        cout << cstream.str() << endl;
        break;
    case ITContainerIter::StateConversionFailed:
        cerr << "Conversion failed, item " << iter.Index() << endl;
        break;
    case ITContainerIter::StateUninitialized:
        cerr << "Iterator was not initialized" << endl;
        break;
        }

    arraycont->Release();                                           // 5 begin
                                                                    // 5 end

    return 0;
}





