/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: csql2.cpp
//  Description:
//	 Submit simple query example with error handling

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iostream.h>
#include <it.h>

int
doneWithSQL(char *sql);

ITCallbackResult                                                    // 1 begin
my_error_handler(const ITErrorManager &errorobject,                 
                 void *userdata,
                 long errorlevel)
{
    // Cast the user data into a stream
    ostream *stream = (ostream *) userdata;
    (*stream) << "my_error_handler: errorlevel="
	      << errorlevel
	      << " sqlstate="
              << errorobject.SqlState()
              << ' '
              << errorobject.ErrorText()
              << endl;
    return IT_NOTHANDLED;
}                                                                   // 1 end

int
main(int, char *)
{
    // Make a connection using defaults	
    ITConnection conn;
    conn.Open();

    // Create a query object
    ITQuery query(conn);

    char qtext[4096];
    char qtext2[4096];

    cout << "> ";

    // Add the callback handler
    query.AddCallback(my_error_handler, (void *) &cerr);            // 2 begin
                                                                    // 2 end
    while (cin.getline(qtext, 1024) && qtext[0] != '.' )
    {
        while (!doneWithSQL(qtext))
	{
	  if (cin.getline(qtext2,1024) && qtext2[0] != '.')
	  {
            strcat (qtext,"\n");
            strcat (qtext,qtext2);
	  } 
	  else 
	  {
	    conn.Close();
	    cout << endl;
	    return 0;
	  }
        }

        if (!query.ExecForIteration(qtext))
        {
            cout << "Could not execute query: " << qtext << endl;
        }
        else
        {
            ITRow *row;
            int rowcount = 0;
            while ((row = query.NextRow()) != NULL)
            {
                rowcount++;
                cout << row->Printable() << endl;
                row->Release();
            }
            cout << rowcount << " rows received, Command:"
                 << query.Command() << endl;
        }
        cout << "> ";
    }
    conn.Close();
    cout << endl;

    return 0;
}

//
// returns true if the SQL is complete otherwise returns false
// if a comment is found then all text until the next line is ignored
// You can have double quotes inside of single quotes and
// single quotes inside of double quotes
//
int
doneWithSQL(char *sql)
{
        enum { Text, Squote, Dquote, Minus, Comment, Done } state;
 
        char *p;
 
        int quoteLevel = 0;
	char *comment = NULL;
 
        state = Text;
 
        int size = strlen(sql);
 
        for (p = sql; size > 0; p += 1, size -= 1)
        {
                switch (*p)
                {
                case ';':
                        if ((state == Text) || (state == Minus))
                                state = Done;
                        break;
                case '\"':
                        if ( state == Dquote )
                                state = --quoteLevel ? Squote : Text;
                        else if (state != Comment)
                                quoteLevel++, state = Dquote;
                        break;
                case '\'':
                        if ( state == Squote )
                                state = --quoteLevel ? Dquote : Text;
                        else if (state != Comment)
                                quoteLevel++, state = Squote;
                        break;
                case '-':
                        // Assume '--' does not finish comment
                        if ( state == Minus )
			{
                                state = Comment;
				comment = p - 1;
			}
                        else if ( state == Text )
                                state = Minus;
                        break;
                case '\n':
                        if ( state == Comment || state == Minus )
                                state = Text;
                        break;
                default:
                        if ( state == Minus )
                                state = Text;
                        break;
                }
        }

	if (comment != NULL)
		*comment = '\0';

        return (state == Done? TRUE : FALSE);
}
