/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: csql3.cpp
//  Description:
//	 Submit simple query example with error handling and
//	 transaction monitoring

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <it.h>

ITCallbackResult
my_error_handler(const ITErrorManager &errorobject,
                 void *userdata,
                 long errorlevel)
{
    // Cast the user data into a stream
    ostream *stream = (ostream *) userdata;
    (*stream) << "my_error_handler: errorlevel="
	      << errorlevel
	      << " sqlstate="
              << errorobject.SqlState()
              << ' '
              << errorobject.ErrorText()
              << endl;
    return IT_NOTHANDLED;
}

int
main(int, char *)
{
    // Make a connection using defaults					
    ITConnection conn;
    conn.Open();

    // Create a query object
    ITQuery query(conn);
    char qtext[1024];

    // Add the callback handler
    query.AddCallback(my_error_handler, (void *) &cerr);

    cout << "> ";
    
    while (cin.getline(qtext, 1024))
    {
        if (!query.ExecForIteration(qtext))
        {
            cout << "Could not execute query: " << qtext << endl;
        }
        else
        {
            ITRow *row;
            int rowcount = 0;
            while ((row = query.NextRow()) != NULL)
            {
                rowcount++;
                cout << row->Printable() << endl;
                row->Release();
            }
            cout << rowcount << " rows received, Command:"
                 << query.Command() << endl;
        }

        if (conn.GetTransactionState() == ITConnection::Begin)      // 1 begin
        {
            cout << "TRANSACTION> ";
        }
        else
        {
            cout << "> ";
        }                                                           // 1 end
    }

    if (conn.GetTransactionState() == ITConnection::Begin)          // 2 begin
    {
        cerr << endl
             << "Exit within transaction, aborting transaction"
             << endl;
        conn.SetTransaction(ITConnection::Abort);
    }                                                               // 2 end
    conn.Close();
    cout << endl;

    return 0;
}
