/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: cursupd.cpp
//
//  Description:
//	 Demonstrates cursor fetch and update functions and ITPositionCurrent use
//   with the cursors.
//

#include <ctype.h>
#include <string.h>
#include <iostream.h>
#include <it.h>

ITCallbackResult
errorHandler(const ITErrorManager &errorobject,                 
                 void *userdata,
                 long errorlevel)
{
    // Cast the user data into a stream
    ostream *stream = (ostream *) userdata;
    (*stream) << "my_error_handler: errorlevel="
	      << errorlevel
	      << " sqlstate="
              << errorobject.SqlState()
              << ' '
              << errorobject.ErrorText()
              << endl;
    return IT_HANDLED;
}

int
main( int, char ** )
{
    ITConnection conn;
    conn.Open();
    ITCursor cursor( conn );                                      // 1 begin
                                                                  // 1 end
    cursor.AddCallback( errorHandler, (void *)&cerr );
    conn.SetTransaction( ITConnection::Begin );                   // 2 begin 

    if( !cursor.Prepare( "select b from bar where b < ?::integer;" ) )
    {                                                             // 2 end
        cerr << "Cursor select prepare failed" << endl;
        return -1;
    }
    ITValue *par = cursor.Param( 0 );                             // 3 begin
    if( !par )                                                    // 3 end
    {
        cerr << "Couldn't get cursor parameter's ITValue *" << endl;
        return -1;
    }
    if( !par->FromPrintable( "3" ) )                              // 4 begin
    {                                                             // 4 end
        cerr << "Couldn't set cursor parameter" << endl;
        return -1;
    }
    par->Release();
    if( !cursor.Open( 0, "bar" ) )                                // 5 begin
    {                                                             // 5 end
        cerr << "Couldn't open cursor" << endl;
        return -1;
    }
    ITRow *row;                                                   // 6 begin
    while( row = cursor.NextRow() ) 
        {
        ITValue *col = row->Column( 0  );                        
        if( !col )                                              
            {
            cerr << "Couldn't get the column from the cursor's row" << endl;
            return -1;
            }                                                  

        cout << "Column 0 was " << col->Printable() << endl;      // 6 end

        ITRow *duprow = cursor.NextRow( 0, ITPositionCurrent );
        if( !duprow )
            {
            cout << "Couldn't re-fetch the row" << endl;
            return -1;
            }
        if( !duprow->Equal( row ) )
            {
            cout << "Two instances of the same row are unequal" << endl;
            return -1;
            }

        ITValue *colduprow = duprow->Column(0);
        if( !colduprow ) 
            {
            cout << "Couldn't get the column from re-fetched row" << endl;
            return -1;
            }

        if( !colduprow->FromPrintable( "2" ) )                  // 7 begin
            {
            cerr << "Couldn't set the column value" << endl;
            return -1;
            }
        else
            {
            cout << "Column 0 is now " << colduprow->Printable() << endl;
            }							// 7 end

        if( duprow->Equal( row ) )
            {
            cout << "Row is not updated" << endl;
            return -1;
            }

        col->Release();
        colduprow->Release();

        if( !cursor.UpdateCurrent() )                             // 8 begin
        {                                                     
            cerr << "Could not update the current row" << endl;
            return -1;
        }                                                         // 8 end

        row->Release();
        duprow->Release();
    }
    conn.SetTransaction( ITConnection::Abort );                  
    cursor.Close();                                               
    conn.Close();
    return 0;       
}
