/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: delegate.cpp
//  Description:
//	 Object delegation example.

#include <stdlib.h>
#include <it.h>
#ifndef MI_WINNT
#include <strstream.h>
#else
#include <strstrea.h>
#endif

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#define NBITS 32

class Bitarray : public ITContainCvt			// 1 begin
{
public:
    // Overrides of ITEssential methods
    virtual ITOpErrorCode IT_STDCALL QueryInterface(const ITInterfaceID &ifiid,
                                                    void **resultif);
    virtual unsigned long IT_STDCALL AddRef();
    virtual unsigned long IT_STDCALL Release();		// 1 end
    
    // Overrides of ITContainCvt methods		// 2 begin
    virtual long IT_STDCALL NumItems();			// 2 end

    // Note that only integer conversions are supported.
    virtual ITBool IT_STDCALL ConvertTo(long, short &)       { return FALSE; }
    virtual ITBool IT_STDCALL ConvertFrom(long, short)       { return FALSE; }
    virtual ITBool IT_STDCALL ConvertTo(long, int &);
    virtual ITBool IT_STDCALL ConvertFrom(long, int);
    virtual ITBool IT_STDCALL ConvertTo(long, long &)        { return FALSE; }
    virtual ITBool IT_STDCALL ConvertFrom(long, long)        { return FALSE; }
    virtual ITBool IT_STDCALL ConvertTo(long, float &)       { return FALSE; }
    virtual ITBool IT_STDCALL ConvertFrom(long, float)       { return FALSE; }
    virtual ITBool IT_STDCALL ConvertTo(long, double &)      { return FALSE; }
    virtual ITBool IT_STDCALL ConvertFrom(long, double)      { return FALSE; }
    virtual ITBool IT_STDCALL ConvertTo(long, long double &) { return FALSE; }
    virtual ITBool IT_STDCALL ConvertFrom(long, long double) { return FALSE; }
    virtual ITBool IT_STDCALL ConvertTo(long, const char * &){ return FALSE; }
    virtual ITBool IT_STDCALL ConvertFrom(long, const char *){ return FALSE; }
    virtual ITBool IT_STDCALL ConvertTo(long, ITString &)    { return FALSE; }
    virtual ITBool IT_STDCALL ConvertFrom(long, const ITString &)
    { return FALSE; }
    virtual ITBool IT_STDCALL ConvertTo(long, ITInt8 &)      { return FALSE; }
    virtual ITBool IT_STDCALL ConvertFrom(long, ITInt8)      { return FALSE; }
#ifdef IT_COMPILER_HAS_BOOL
    virtual ITBool IT_STDCALL ConvertTo(long, bool &)        { return FALSE; }
    virtual ITBool IT_STDCALL ConvertFrom(long, bool)        { return FALSE; }
#endif

    Bitarray( ITMVDesc * );
    ~Bitarray();
    static ITValue *MakeValue(ITMVDesc *);

    ITValue *int_value;						// 4 begin
								// 4 end
    // The interface is passed back, but we keep the real ITEssential
    ITEssential *int_essential;                                     // 3 begin
                                                                    // 3 end
    ITTypeInfo typeinfo;
    long refcount;
};

Bitarray::Bitarray( ITMVDesc *mv )
    : refcount( 1 ), typeinfo( *mv->vf_origtypeinfo )
{    
    ITMVDesc desc = *mv;					// 6 begin
    desc.vf_origtypeinfo = (ITTypeInfo *)mv->vf_origtypeinfo->Source(); // 6 end
    desc.vf_outerunknown = this;			// 5 begin
									// 5 end
    int_value = ITFactoryList::DatumToValue( desc );
    int_essential = desc.vf_outerunknown;	// 7 begin
									// 7 end
    mv->vf_outerunknown = this;
}

Bitarray::~Bitarray()
{
    int_essential->Release();	// 8 begin
						// 8 end
}


ITValue *
Bitarray::MakeValue( ITMVDesc *mv )
{
    Bitarray *p = new Bitarray( mv );
    ITValue *pv;
    p->QueryInterface( ITValueIID, (void **)&pv );
    return pv;
}

// Overrides
ITOpErrorCode                                                      // 9 begin
Bitarray::QueryInterface(const ITInterfaceID &iid,                
			  void **ifptr)
{
    switch (ITIIDtoSID(iid))
        {
    case ITEssentialSID: case ITContainCvtSID:
        *ifptr = this;
        AddRef();
        return IT_QUERYINTERFACE_SUCCESS;
        
    default:
        // This object does not support the interface. Try the
        // delegated sub-object...if the sub-object supports the
        // interface, it will increment the reference counter on the
        // controlling unknown, so we don't need to increment it
        // here (except if you ask the sub-object for its ITEssential
        // interface, in which case it will increment its own
        // reference count).
        return int_essential->QueryInterface(iid, ifptr);
        }
}                                                                  // 9 end

unsigned long
Bitarray::AddRef()
{
    return ++refcount;
}

unsigned long
Bitarray::Release()
{
    if (--refcount <= 0)
    {
        delete this;
        return 0;
    }
    else
    {
        return refcount;
    }
}

// ContainCvt implementation					// 10 begin
ITBool                   
Bitarray::ConvertTo(long item, int &dbvalue)
{
    if ( int_value->IsNull() || item >= NumItems())
        return FALSE;
    const char *valasstr = int_value->Printable();
    int val = atoi( valasstr );
    dbvalue = !!(val & (1 << (NBITS - 1 - item)));
    return TRUE;
}                        

ITBool
Bitarray::ConvertFrom( long item, int val )
{
    if( NumItems() <= item )
        return FALSE;
    int value = val ? value | (1 << (NBITS - 1 - item))
        : value & ~(1 << (NBITS - 1 - item));
    char valasstr[32];
    sprintf( valasstr, "%d", value );
    return int_value->FromPrintable( valasstr );
}

long
Bitarray::NumItems()
{
  return NBITS;
}								// 10 end

ITFactoryList BitarrayFactory( "bitarray", Bitarray::MakeValue );
