/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: loadtab.cpp
//
//  Description:
//	 Prepared statement example. 
//
// Parameters:
//	table_name  - the table you want to populate
//	column_name(s) - the names of the columns in the table to populate
//
// Notes:
//	table_name and column_name(s) are optional
//	If they are omitted, then the default parms take effect
//      See below for details
//

#include <ctype.h>
#include <string.h>
#include <iostream.h>
#include <it.h>

ITCallbackResult
errorHandler(const ITErrorManager &errorobject,                 
                 void *userdata,
                 long errorlevel)
{
    // Cast the user data into a stream
    cerr << "my_error_handler: connstr="
         << (char *) userdata
         << " errorlevel="
         << errorlevel
         << (errorobject.Warn() == TRUE? 
		"\tALWAYS_DIFFERENT Warning" : " Error")
         << " sqlstate="
         << errorobject.SqlState()
         << ' '
         << (errorobject.Warn() == TRUE?
		errorobject.WarningText() : errorobject.ErrorText())
         << endl;

    return IT_NOTHANDLED;
}

int
main( int argc, char **argv )
{
    int paramno;

    if( argc == 2 )
        {
        cerr 
            << "Usage:\n\t" 
            << argv[0] 
            << " [table_name column_name(s)]" 
            << endl;
        return -1;
        }

    if( 0 < strcmp( IT_VERSION, ITVersion() ) )
        {
        cerr << "C++ Interface version mismatch" << endl;
        return -1;
        }

    // Connect to default and make a statement. Attach errorHandler.
    ITConnection conn;
    conn.Open();
    ITStatement stmt( conn );                                      // 1 begin
                                                                   // 1 end
    stmt.AddCallback( errorHandler, (void *)&cerr );

    // Build and prepare insert SQL.
    ITString sql = "insert into ";

    if (argc >= 3) 
    {
      sql.Append( argv[1] );
      sql.Append( "(" );
      int colnameno;
    
      for( colnameno = 0; colnameno < argc - 2; colnameno++ )
      {
          sql.Append( argv[colnameno + 2] );
          sql.Append( "," );
      }

      sql.Trim( "," );
      sql.Append( ") values (" );
      for( colnameno = 0; colnameno < argc - 2; colnameno++ )
          sql.Append( "?," );

      sql.Trim( "," );
      sql.Append( ");" );
    } 
    else 
    {
      sql.Append( "regresstab ( "\
		"boolean_val, "\
		"char_val, "\
		"character_val, "\
		"date_val, "\
		"datetime_val, "\
		"dec_val, "\
		"decimal_val, "\
		"double_val, "\
		"float_val, "\
		"int_val, "\
		"int8_val, "\
		"integer_val, "\
		"interval_ds_val, "\
		"interval_ym_val, "\
		"lvarchar_val, "\
		"numeric_val, "\
		"money_val, "\
		"real_val, "\
		"serial_val, "\
		"smallfloat_val, "\
		"smallint_val, "\
		"text_val, "\
		"byte_val, "\
		"varchar_val, "\
		"large_object_val, "\
		"large_text_val, "\
		"boolean_null, "\
		"char_null, "\
		"character_null, "\
		"date_null, "\
		"datetime_null, "\
		"dec_null, "\
		"decimal_null, "\
		"double_null, "\
		"float_null, "\
		"int_null, "\
		"int8_null, "\
		"integer_null, "\
		"interval_ds_null, "\
		"interval_ym_null, "\
		"lvarchar_null, "\
		"numeric_null, "\
		"money_null, "\
		"real_null, "\
		"smallfloat_null, "\
		"smallint_null, "\
		"text_null, "\
		"varchar_null "\
		") "\
		"values ( "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"?, "\
		"? "\
		");");
    }

    cout << "Preparing the following SQL: " << sql << endl;

    if( !stmt.Prepare( sql ) )                                    // 2 begin
        return -1;                                                // 2 end

    // Read cin, set parameters and execute statement.
    // Data file consists of white space - separated columns.
    // Each column either contains a double-quoted string
    // (quotes are repeated inside the string) or 
    // a sequence of non white space characters.
    // NULL is represented by non-quoted word NULL

    int nparams = stmt.NumParams();

    cout << "There are " << nparams << " parameter markers in the SQL" << endl;

    char db[32767]; // row data buffer

    // each input line has one row of parms
    while( cin.getline( db, sizeof db ) )
    {
        int dbpos = 0;

        // exec in a transaction since might have to create smartblob
        conn.SetTransaction( ITConnection::Begin );

        for (  paramno = 0; paramno < nparams; paramno++ )
        {
            // ignore leading spaces
            while ( isspace( db[dbpos] ) )
                dbpos++;

            char pdb[256]; // parameter data buffer
            int pdbpos = 0;
            ITBool null = FALSE;

            // found beginning of quote, now let's look for end
            if ( db[dbpos] == '"')
            {
                // position to first char
                dbpos++;
                ITBool dquote;

                    // copy char and advance source and target buffers
                    pdb[pdbpos++] = db[dbpos++];

                    while ( (db[dbpos] != '\0') && (db[dbpos] != '"') )
                    {
                        if ( pdbpos == sizeof pdb )
                        {
                            cerr << "Parameter buffer overflow" << endl;
                            return -1;
                        }

                        // copy next char and advance source and target buffers
                        pdb[pdbpos++] = db[dbpos++];
                    }

                    if ( db[dbpos] != '"' )
                    {
                        cerr << "Missing closing quote" << endl;
                        return -1;
                    }
                    else
                    {
                        // advance passed closing quote
                        dbpos++;
                    }
            }
            else
            {
                while ( (db[dbpos] != '\0') && (!isspace(db[dbpos])) )
                    if ( pdbpos == sizeof pdb )
                    {
                        cerr << "Parameter buffer overflow" << endl;
                        return -1;
                    }
                    else
                    {
                        pdb[pdbpos++] = db[dbpos++];
                    }
                if ( !strncmp( "NULL", pdb, 4 ) )
                    null = TRUE;
            }

            pdb[pdbpos] = '\0';

            cout << "Parameter " << paramno << " is '" << pdb << "'" << endl;

            ITValue *param = stmt.Param( paramno );                // 3 begin
                                                                   // 3 end
            if ( !param )
                {
                cerr << "Couldn't get parameter " << paramno << endl;
                return -1;
                }

            if (param->TypeOf().IsCollection())
                {
				cerr 
					<< "Param " 
					<< paramno 
					<< " type "
					<< param->TypeOf().Name()
					<< " is unsupported with FromPrintable() yet" 
					<< endl;
                return -1;
                }
			else if ( (param->TypeOf().Name().Equal("blob"))
                         ||   (param->TypeOf().Name().Equal("clob")) )
                {
                ITLargeObjectManager lobMgr(conn);
                lobMgr.CreateLO();
                lobMgr.Write(pdb, strlen(pdb));
                        
                if (!param->FromPrintable(lobMgr.HandleText()))
                    {
                    cerr 
                        << "Could not set LOB parameter " 
                        << paramno << " to '" << pdb << "'" << endl;
                    return -1;
                    }
                }
            else if( param->TypeOf().Name().Equal( "byte" ) )
                {
                ITDatum *pdatum = 0;
                param->QueryInterface( ITDatumIID, (void **)&pdatum );
                if( !pdatum )
                    {
                    cerr << "BYTE type does not expose ITDatum???" << endl;
                    return -1;
                    }
                if( !pdatum->SetData( pdb, pdbpos, 0 ) )
                    {
                    cerr << "SetData() for BYTE failed" << endl;
                    return -1;
                    }
                pdatum->Release();
                }
            else if ( null == TRUE )
                { 
                if ( !param->SetNull() )
				    {
					cerr << "Could not set parameter " 
                         << paramno << " to null" << endl;
                    return -1;
				    }
                }
            else
                if ( !param->FromPrintable( pdb ) )                // 4 begin
                    {
                    cerr << "Could not set parameter " 
                         << paramno << " to '" << pdb << "'" << endl;
                    return -1;                                     // 4 end
                    }

            param->Release();

        }

        if ((db[dbpos] == '\0') && (paramno < nparams))
        {
            cerr << "Error - Only " << paramno+1 
                 << " parameter(s) supplied out of "
                 << nparams << " required" << endl;
            return -1;
        }

        if ( !stmt.Exec() )                                        // 5 begin
		{
			cerr << "Could not execute statement" << endl;
            return -1;                                             // 5 end
		}

    	cout << "Number of rows affected was: " << stmt.RowCount() << endl;

        conn.SetTransaction( ITConnection::Commit );
    }

    return 0;
}
