/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: loex1.cpp
//
//  Description:
//	 Large object example

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <it.h>

#ifdef IT_WINDOWS
#include <fcntl.h>
#include <io.h>
#endif

ITCallbackResult
my_error_handler(const ITErrorManager &errorobject,
                 void *userdata,
                 long errorlevel)
{
    // Cast the user data into a stream
    ostream *stream = (ostream *) userdata;
    (*stream) << "my_error_handler: errorlevel="
	      << errorlevel
	      << " sqlstate="
              << errorobject.SqlState()
              << ' '
              << errorobject.ErrorText()
              << endl;
    return IT_NOTHANDLED;
}

int
main(int, char **)
{
    // Make a connection using defaults
    ITConnection conn;
    conn.Open();

    // Create a query object
    ITQuery q(conn);

    // Add the callback handler
    q.AddCallback(my_error_handler, (void *) &cerr);

    // Retrieve a row from the database
    ITRow *row;           
    row = q.ExecOneRow("select large_object_val from regresstab;");

    // Check to see if any rows were returned
    if (row == NULL)
    {
        cout << "loex1: no rows returned by ITQuery::ExecOneRow" << endl;
        cout << "loex1: exiting" << endl;
        return 1;
    }

    ITValue *v = row->Column(0);
    ITLargeObject *loif;                                            // 1 begin
                                                                    // 1 end
    // Extract an interface. The return code IT_QUERYINTERFACE_SUCCESS
    // should be used for compatibility reasons.
    if (v->QueryInterface(ITLargeObjectIID, (void **) &loif)        // 1 begin
        == IT_QUERYINTERFACE_SUCCESS)                               // 1 end
    {
        long n;
        char buf[8192];
        
        // Windows will change \n to \r\n if one doesn't set the
        // mode of the file to binary.
        
#ifdef IT_WINDOWS
        _setmode(_fileno(stdout), _O_BINARY);
#endif

        while ((n = loif->Read(buf, sizeof(buf))) > 0)              // 2 begin
        {
            cout.write(buf, n);
        }                                                           
        cout.flush();                                               // 2 end

#ifdef IT_WINDOWS
        _setmode(_fileno(stdout), _O_TEXT);
#endif
        cout << endl << "EOF" << endl;

        // Release the large object interface
        loif->Release();
    }

    // Release the value interfaces
    v->Release();
    row->Release();

    return 0;
}


