/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: rawval.cpp
//
//  Description:
//	 Example of raw data access
//

#include <iostream.h>
#include <it.h>

#include <mitrace.h>
#include <string.h>

int
main(int argc, char *argv[])
{
    // Use libdmi tracing
    if (argc == 2)
    {
        mi_trace(argv[1]);
    }

    // Use defaults to open connection
    ITConnection conn;
    conn.Open();

    if (conn.Error())
    {
        cerr << conn.ErrorText() << endl;
        return 1;
    }
    
    ITQuery q(conn);                                                // 1 begin

    ITRow *row =                                                    
        q.ExecOneRow("select byte_val from regresstab;");           // 1 end

    if (q.Error())
    {
        cerr << q.ErrorText() << endl;
        return 1;
    }

    // Check to see if any rows were returned
    if (row == NULL)
    {
        cout << "rawval: no rows returned by ITQuery::ExecOneRow" << endl;
        cout << "rawval: exiting" << endl;
        return 1;
    }

    // Extract the column                                           // 1 begin
    ITValue *v;
    v = row->Column(0);                                             // 1 end

    // Extract out the interface from the value that 
    // provides access to its raw data.
    ITDatum *rv;                                                    // 2 begin
    if (v->QueryInterface(ITDatumIID, (void **) &rv) ==
        IT_QUERYINTERFACE_SUCCESS)
        {                                                           // 2 end
        
        cout 
            << "The number of bytes in the byte array is: " 
            << rv->DataLength() 
                << endl;


        char *pch = (char *)rv->Data();                             // 3 begin
                                                                    // 3 end

        // Check to see if any data returned by ITDatum::Data
        if (pch == NULL)
        {
            cout << "rawval: no data returned by ITDatum::Data" << endl;
            cout << "rawval: exiting" << endl;
            return 1;
        }        

        char match[] = "Informix";                                  // 4 begin
        
        char *found = strstr(pch, match);                           // 4 end

        if (found)
            cout 
                << "Found '" 
                << match 
                << "' at position " 
                << found - pch 
                << endl;
        else
            cout 
                << "Could not find '"
                << match
                << "' in the byte array" 
                << endl
                << "----------------------------------------------------------"
                << endl
                << pch
                << endl
                << "----------------------------------------------------------"
                << endl;

        // Release the interface
        rv->Release();                                              // 5 begin
                                                                    // 5 end
        }                                                               
    else
        {
        cout 
            << "Couldn't get ITDatum interface from value object - Error" 
            << endl;
        }
                                                                    
    // Release the value interface
    v->Release();
    
    // Release the row
    row->Release();
    
    return 0;
}
