/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: tabcnt.cpp
//
//  Description:
//	 Simple example of issuing query and using value interfaces

#include <iostream.h>
#include <it.h>

int
main(int, char **)
{
    // Make a connection using defaults
    ITConnection conn;
    conn.Open();

    // Create a query object
    ITQuery q(conn);

    // Retrieve a row from the database
    ITRow *row;                                                     // 1 begin
    row = q.ExecOneRow("select unique count(*) from systables where tabname in ('systables', 'syscolumns', 'sysviews');"); // 1 end


    // Check to see if any rows were returned
    if (row == NULL)
    {
        cout << "tabcnt: no rows returned by ITQuery::ExecOneRow" << endl;
        cout << "tabcnt: exiting" << endl;
        return 1;
    }

    ITValue *v = row->Column(0);                                    // 2 begin
                                                                    // 2 end
    ITConversions *c;                                               // 3 begin

    // Extract an interface. The return code IT_QUERYINTERFACE_SUCCESS
    // should be used for compatibility reasons.
    if (v->QueryInterface(ITConversionsIID, (void **) &c)           
        == IT_QUERYINTERFACE_SUCCESS)
    {                                                               // 3 end
        int numtabs;                                                // 4 begin
        if (c->ConvertTo(numtabs))
        {
            cout << "Number of rows in the query was: " << numtabs << endl;
        }
        
        // Release the conversions interface
        c->Release();                                               // 4 end
    }

    // Release the value interfaces
    v->Release();                                                   // 5 begin
    row->Release();                                                 // 5 end

    return 0;
}

