/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: testtype.cpp

#include <it.h>

#ifndef MI_WINNT
#include <strstream.h>
#else
#include <strstrea.h>
#endif

// For dynamically loaded libraries this must be a C entry point for
// the dynamic loader to find the symbol
extern "C" ITValue *MakeValue(ITMVDesc *mvdesc);

class TestType
{
public:
    class XITDatum : public ITDatum
    {
    public:
        // Overrides of ITEssential methods
        virtual ITOpErrorCode IT_STDCALL QueryInterface
			(const ITInterfaceID &ifiid, void **resultif);
        virtual unsigned long IT_STDCALL AddRef();
        virtual unsigned long IT_STDCALL Release();

        // Overrides of ITValue methods
        virtual const ITString & IT_STDCALL Printable();
        virtual const ITTypeInfo & IT_STDCALL TypeOf();
        virtual ITBool IT_STDCALL IsNull();
        virtual ITBool IT_STDCALL SameType(ITValue *);
        virtual ITBool IT_STDCALL CompatibleType(ITValue *);
        virtual ITBool IT_STDCALL Equal(ITValue *);
        virtual ITBool IT_STDCALL LessThan(ITValue *);
        virtual ITBool IT_STDCALL IsUpdated();
        virtual ITBool IT_STDCALL FromPrintable( const ITString & );
        virtual ITBool IT_STDCALL SetNull();
 
        // ITDatum methods
        virtual MI_DATUM     IT_STDCALL Data();
        virtual long         IT_STDCALL DataLength();
        virtual ITBool       IT_STDCALL SetData
            (MI_DATUM data, long dataLen, ITPreserveData *preservedata);
        virtual const ITConnection& IT_STDCALL Connection();
 
        TestType *parent;
    } datum_interface;

    TestType(ITMVDesc *mv);
    ~TestType();

    // Data members to implement ITEssential functionality
    long refcount;

    // Data members to implement ITValue functionality
    ITTypeInfo typeinfo;
    ITBool isnull, isupdated;
    ITString printable_value;

    // Value...testtype is a pseudonym for 'integer'
    long value;

    // Real implementations of overrides
    virtual ITOpErrorCode IT_STDCALL QueryInterface(const ITInterfaceID &iid, void **ifptr);
    virtual unsigned long IT_STDCALL AddRef();
    virtual unsigned long IT_STDCALL Release();

    virtual const ITString & IT_STDCALL Printable();
    virtual const ITTypeInfo & IT_STDCALL TypeOf();
    virtual ITBool IT_STDCALL IsNull();
    virtual ITBool IT_STDCALL SameType(ITValue *);
    virtual ITBool IT_STDCALL CompatibleType(ITValue *);
    virtual ITBool IT_STDCALL Equal(ITValue *);
    virtual ITBool IT_STDCALL LessThan(ITValue *);
    virtual ITBool IT_STDCALL IsUpdated();
    virtual ITBool IT_STDCALL FromPrintable( const ITString & );
    virtual ITBool IT_STDCALL SetNull();

    // Datum implementations
    virtual MI_DATUM            IT_STDCALL Data();
    virtual long                IT_STDCALL DataLength();
    virtual ITBool              IT_STDCALL SetData
        (MI_DATUM data, long dataLen, ITPreserveData *preservedata);
    virtual const ITConnection& IT_STDCALL Connection();

    ITConnection conn;
};

ITValue *
MakeValue(ITMVDesc *mv)
{
    TestType *impl = new TestType(mv);
    return (ITValue *)&impl->datum_interface;
}

// Implementation
TestType::TestType(ITMVDesc *mv)
    : refcount(1), typeinfo(*mv->vf_origtypeinfo), isupdated( FALSE ),
      conn(*mv->vf_connection)
{
    // Copy data
    value = *(mi_integer *) mv->vf_data;

    // NULL?
    isnull = mv->vf_libmivaluetype == MI_NULL_VALUE;

    // set up interfaces
    datum_interface.parent = this;
}

TestType::~TestType()
{}

// Overrides

ITOpErrorCode
TestType::QueryInterface(const ITInterfaceID &iid,
                                void **ifptr)
{
    int result = IT_QUERYINTERFACE_SUCCESS;
    
    // Return different interfaces as appropriate by referencing
    // nested class members.
    switch (ITIIDtoSID(iid))
    {
        case ITEssentialSID:
        case ITValueSID:
        case ITDatumSID:
            *ifptr = (void *) &datum_interface;
            break;

        default:
            result = IT_QUERYINTERFACE_FAILED;
            *ifptr = NULL;
            break;
    }
    if (result == IT_QUERYINTERFACE_SUCCESS)
        AddRef();

    return result;
}

unsigned long
TestType::AddRef()
{
    return ++refcount;
}

unsigned long
TestType::Release()
{
    if (--refcount <= 0)
    {
        delete this;
        return 0;
    }
    else
    {
        return refcount;
    }
}

// Overrides of ITValue methods

// Printable: return the integer rendered into a string
const ITString &
TestType::Printable()
{
    if( IsNull() )
        return printable_value = "null";
    ostrstream str;
    str << "0x" << hex  << value << ends;
    char *tmp = str.str();
    printable_value = tmp;
    delete tmp;
    return printable_value;
}

const ITTypeInfo &
TestType::TypeOf()
{
    return typeinfo;
}

ITBool
TestType::IsNull()
{
    return isnull;
}

ITBool
TestType::SameType(ITValue *v)
{
    return TypeOf().Name().Equal(v->TypeOf().Name());
}

ITBool
TestType::CompatibleType(ITValue *)
{
    return TRUE;
}

ITBool
TestType::Equal(ITValue *v)
{
    return IsNull() ? FALSE : Printable().Equal(v->Printable());
}

ITBool
TestType::LessThan(ITValue *)
{
    return FALSE;
}

ITBool
TestType::IsUpdated()
{
    return isupdated;
}

ITBool
TestType::FromPrintable( const ITString &v )
{
    if( v.IsNull() )
        isnull = TRUE;
    else
        {
        istrstream str( (char *)v.Data(), v.Size() );
        str >> value;
        isnull = FALSE;
        }
    return isupdated = TRUE;
}

ITBool
TestType::SetNull()
{
    isnull = TRUE;
    isupdated = TRUE;
    return TRUE;
}

// ITDatum implementations
 
MI_DATUM
TestType::Data()
{
    return &value;
}
 
long
TestType::DataLength()
{
    return sizeof value;
}
 
ITBool
TestType::SetData(MI_DATUM data, long datalen, ITPreserveData *preservedata)
{
    value = *((long *) data);
    isnull = FALSE;
    return isupdated = TRUE;
}
 
const ITConnection&
TestType::Connection()
{
    return conn;
}

// NESTED CLASS PASSTHROUGHS
ITOpErrorCode
TestType::XITDatum::QueryInterface(const ITInterfaceID &ifiid,
                                          void **resultif)
{
    return parent->QueryInterface(ifiid, resultif);
}

unsigned long
TestType::XITDatum::AddRef()
{
    return parent->AddRef();
}

unsigned long
TestType::XITDatum::Release()
{
    return parent->Release();
}

const ITString &
TestType::XITDatum::Printable()
{
    return parent->Printable();
}

const ITTypeInfo &
TestType::XITDatum::TypeOf()
{
    return parent->TypeOf();
}

ITBool
TestType::XITDatum::IsNull()
{
    return parent->IsNull();
}

ITBool
TestType::XITDatum::SameType(ITValue *v)
{
    return parent->SameType(v);
}

ITBool
TestType::XITDatum::CompatibleType(ITValue *v)
{
    return parent->CompatibleType(v);
}

ITBool
TestType::XITDatum::Equal(ITValue *v)
{
    return parent->Equal(v);
}

ITBool
TestType::XITDatum::LessThan(ITValue *v)
{
    return parent->LessThan(v);
}

ITBool
TestType::XITDatum::IsUpdated()
{
    return parent->IsUpdated();
}

ITBool
TestType::XITDatum::FromPrintable( const ITString &v )
{
    return parent->FromPrintable( v );
}

ITBool
TestType::XITDatum::SetNull()
{
    return parent->SetNull();
}

MI_DATUM
TestType::XITDatum::Data()
{
    return parent->Data();
}
 
long
TestType::XITDatum::DataLength()
{
    return parent->DataLength();
}
 
ITBool
TestType::XITDatum::SetData
(MI_DATUM data, long datalen, ITPreserveData *preservedata)
{
    return parent->SetData(data, datalen, preservedata);
}
 
const ITConnection&
TestType::XITDatum::Connection()
{
    return parent->Connection();
}

